/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;

/**
 * Abstrakte Implementation eines Informationselementes einer TIC3-Meldung.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: AbstractTIC3InfoElement.java 63650 2016-09-22 17:11:32Z gieseler $
 */

public abstract class AbstractTIC3InfoElement implements TIC3InfoElement {

	/**
	 * Der zum Element geh&ouml;rende Name des XML-Elementes.
	 */
	private String xmlElementName = "";

	/**
	 * Der Name des zum Element geh&ouml;renden DAV-Attributes.
	 */
	private String davAttributName = "";

	/**
	 * Liste der Kind-Elemente.
	 */
	private List<TIC3InfoElement> kindElemente = new ArrayList<>();

	/**
	 * Liste der Attribute.
	 */
	private List<TIC3InfoAttribut> attribute = new ArrayList<>();

	/**
	 * 'name'-Attribut.
	 */
	private final TIC3InfoAttribut nameAttribut = new TIC3InfoAttribut("name", null);

	/**
	 * Legt fest, ob der Wert des Attributes 'value' als Wert benutzt werden
	 * soll (anstatt des Element-Wertes).
	 */
	private boolean useAttributValue;

	private TIC3InfoAttribut wertAttribut;

	/**
	 * Gibt das 'name'-Attribut zur&uuml;ck.
	 * 
	 * @return 'name'-Attribut
	 */
	public TIC3InfoAttribut getNameAttribut() {
		return nameAttribut;
	}

	/**
	 * Gibt die Liste der Kind-Elemente zur&uuml;ck.
	 * 
	 * @return Liste der Kind-Elemente
	 */
	public List<TIC3InfoElement> getKindElemente() {
		return kindElemente;
	}

	/**
	 * Erzeugt ein Element.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public AbstractTIC3InfoElement(final String xmlName, final String davName) {
		super();
		this.xmlElementName = xmlName;
		this.davAttributName = davName;
		addAttribut(nameAttribut);
	}

	/**
	 * Erzeugt ein Element.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 * @param attName
	 *            Name des XML-Attributes
	 */
	public AbstractTIC3InfoElement(final String xmlName, final String davName, final String attName) {
		this(xmlName, davName);
		wertAttribut = new TIC3InfoAttribut(attName, davName);
		addAttribut(wertAttribut);
	}

	/**
	 * F&uuml;gt ein Kind-Element hinzu.
	 *
	 * @param child
	 *            Kind-Element
	 */
	public void addKindElement(final TIC3InfoElement child) {
		kindElemente.add(child);
	}

	/**
	 * F&uuml;gt ein Attribut hinzu.
	 *
	 * @param xmlName
	 *            Name des Attributes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public void addAttribut(final String xmlName, final String davName) {
		attribute.add(new TIC3InfoAttribut(xmlName, davName));
	}

	/**
	 * F&uuml;gt ein Attribut hinzu.
	 *
	 * @param attr
	 *            Attribute
	 */
	public void addAttribut(final TIC3InfoAttribut attr) {
		attribute.add(attr);
	}

	@Override
	public void einlesenXML(final Node node) throws TIC3Exception {
		for (TIC3InfoAttribut attr : attribute) {
			attr.einlesenXML(node);
		}

		getXmlDaten(node);

		if (kindElemente.size() > 0) {
			final NodeList nl = node.getChildNodes();
			for (int i = 0; i < nl.getLength(); i++) {
				final Node n = nl.item(i);
				for (final TIC3InfoElement child : kindElemente) {
					if (child.hatNodeHandler(n)) {
						child.einlesenXML(n);
					}
				}
			}
		}
	}

	@Override
	public Element erzeugeDocumentElement(final Document doc, final Element parent) throws TIC3Exception {
		final Element element = doc.createElement(getXmlElementName());
		parent.appendChild(element);
		setXmlDaten(element);

		for (final TIC3InfoElement child : kindElemente) {
			child.erzeugeDocumentElement(doc, element);
		}

		return element;
	}

	/**
	 * Gibt den zum Element geh&ouml;renden Namen des DAV-Attributes
	 * zur&uuml;ck.
	 *
	 * @return Name des DAV-Attributes
	 */
	protected String getDavAttributName() {
		return davAttributName;
	}

	@Override
	public void getDavDaten(final Data davdata) throws TIC3Exception {
		/** keine eigenen Daten */
	}

	@Override
	public void getXmlDaten(final Node node) throws TIC3Exception {
		/** keine eigenen Daten */
	}

	/**
	 * Gibt den zum Element geh&ouml;rende XML-Element Namen zur&uuml;ck.
	 *
	 * @return XML-Element Name
	 */
	@Override
	public String getXmlElementName() {
		return xmlElementName;
	}

	/**
	 * Test, ob das Objekt einen Handler f&uuml;r einen XML-Dokument-Knoten
	 * besitzt.
	 *
	 * @param node
	 *            ein XML-Dokument-Knoten
	 * @return true, wenn das Objekt einen Handler f&uuml;r einen
	 *         XML-Dokument-Knoten besitzt, sonst false
	 */
	@Override
	public boolean hatNodeHandler(final Node node) {
		return node.getNodeName().equals(xmlElementName);
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		setDavDaten(davdata);
		final Data ddata = getLocalDavData(davdata);

		for (final TIC3InfoElement child : kindElemente) {
			child.publizieren(ddata, parameter);
		}

		for (final TIC3InfoAttribut child : attribute) {
			child.publizieren(ddata, parameter);
		}
	}

	public Data getLocalDavData(final Data davdata) throws TIC3Exception {
		Data ddata;
		if ((getDavAttributName() != null) && (getDavAttributName().length() > 0)) {
			ddata = davdata.getItem(getDavAttributName());
			if (ddata.isArray()) {
				final Array array = ddata.asArray();
				array.setLength(1);
				ddata = array.getItem(0);
			}
		} else {
			ddata = davdata;
		}

		return ddata;
	}

	@Override
	public void setDavDaten(final Data davdata) throws TIC3Exception {
		/** keine eigenen Daten */
	}

	@Override
	public void setDavDaten(final Data davdata, final String itemname) throws TIC3Exception {
		/** keine eigenen Daten */
	}

	@Override
	public void setXmlDaten(final Element element) throws TIC3Exception {
		/** keine eigenen Daten */
	}

	public boolean useAttributValue() {
		return wertAttribut != null;
	}

	public TIC3InfoAttribut getWertAttribut() {
		return wertAttribut;
	}

	public List<TIC3InfoAttribut> getAttribute() {
		return attribute;
	}
}
